// stdafx.h : include file for standard system include files,
//  or project specific include files that are used frequently, but
//      are changed infrequently
//

#if !defined(AFX_STDAFX_H__B71FBBBC_81CB_47B5_A555_53A45CFD74F3__INCLUDED_)
#define AFX_STDAFX_H__B71FBBBC_81CB_47B5_A555_53A45CFD74F3__INCLUDED_

#ifdef _MSC_VER
#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
#endif // _MSC_VER

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "mpc_dec.h"
#include "requant.h"


//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_STDAFX_H__B71FBBBC_81CB_47B5_A555_53A45CFD74F3__INCLUDED_)
